import { Lexer } from "chevrotain";
import { CronParser as DefaultParser, JenkinsParser, QuartzParser } from "@/parser";
import { CronVisitor, JenkinsVisitor, QuartzVisitor } from "@/semantic";
import { cronTokens, cronVocabulary, jenkinsTokens, quartzTokens } from "@/lexer";
import CronixMode from "./CronixMode";
/**
 * Convert an expression to its string representation. If the provided expression is already a string it is simply returned.
 * The conversion strategy depends on the options passed (for instance, Quartz has additional fields which require a different processing).
 * @param expression The expression to convert
 * @param options The context to infer the conversion strategy from
 */
export function convertToString(expression, _a) {
    var mode = _a.mode;
    if (typeof expression === "string") {
        return expression;
    }
    // default values to * to reduce boilerplate for the user
    var base = [
        expression.minute || "*",
        expression.hour || "*",
        expression.dayOfMonth || "*",
        expression.month || "*",
        expression.dayOfWeek || "*"
    ];
    switch (mode) {
        case CronixMode.QUARTZ:
            var expr = [expression.second || "0"].concat(base);
            if (expression.year) {
                expr.push(expression.year);
            }
            return expr.join(" ");
        case CronixMode.JENKINS:
        case CronixMode.CRONTAB:
        default:
            return base.join(" ");
    }
}
/**
 * A Cron parser with support for multiple cron dialects.
 */
var CronixParser = /** @class */ (function () {
    function CronixParser(options) {
        if (options === void 0) { options = { mode: CronixMode.CRONTAB }; }
        this.mode = options.mode;
        switch (options.mode) {
            case CronixMode.JENKINS:
                this._lexer = new Lexer(jenkinsTokens);
                this._parser = new JenkinsParser();
                this._visitor = new JenkinsVisitor();
                break;
            case CronixMode.QUARTZ:
                this._lexer = new Lexer(quartzTokens);
                this._parser = new QuartzParser();
                this._visitor = new QuartzVisitor();
                break;
            case CronixMode.CRONTAB:
            default:
                this._lexer = new Lexer(cronTokens);
                this._parser = new DefaultParser(cronVocabulary);
                this._visitor = new CronVisitor();
        }
        this.parse = this.parse.bind(this);
        this.parseField = this.parseField.bind(this);
    }
    /**
     * Parse a complete Cron expression, according to the parser's context.
     * @param expression The expression to parse. Can either be a string or an object describing the expression
     * @return The parsed expression as a syntax tree
     */
    CronixParser.prototype.parse = function (expression) {
        var stringExpr = convertToString(expression, { mode: this.mode });
        this._parser.input = this._lexer.tokenize(stringExpr).tokens;
        var parsed = this._parser.cronExpression();
        return this._visitor.visit(parsed);
    };
    /**
     * Parse an expression element, according to the parser's context.
     * @param expression The expression to parse.
     * @return The parsed expression as a syntax tree
     */
    CronixParser.prototype.parseField = function (expression) {
        this._parser.input = this._lexer.tokenize(expression).tokens;
        var parsed = this._parser.expression();
        return this._visitor.visit(parsed);
    };
    Object.defineProperty(CronixParser.prototype, "lexer", {
        get: function () {
            return this._parser;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CronixParser.prototype, "parser", {
        get: function () {
            return this._parser;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CronixParser.prototype, "visitor", {
        get: function () {
            return this._visitor;
        },
        enumerable: true,
        configurable: true
    });
    return CronixParser;
}());
export default CronixParser;
